#!/bin/bash


## PACKMGN ##
## < packmgn.sh > management tool for install and uninstall *.tgz packages
## on any Linux box without "proprietary" package manager (apt-get, RPM,
## pacman, pkgtool...)

## COPYRIGHT 2016-2017 Free Software Foundation, Inc. License GNU.
## GPL version 3 or later <http://gnu.org/licenses/gpl.html>.
## This is a free software: you are free to use, change and redistribute it.
## There is NO WARRANTY, to the extent permitted by law.
## Beta software, use it as under your own responsabilty !
## Frank Baudet <http://marinux.tuxfamily.org>.

## HISTORY
## 17/04/21 first public version > V0.1

########################################################################


                             ############
                             ### HELP ###
                             ############

disp_help() {

   help='
 ¤
 ¤ ######################## PACKMGN V0.1 HELP ##########################
 ¤
 ¤ < packmgn.sh > simple management tool for install and uninstall
 ¤ *.tar.gz or -identical- *.tgz packages on any Linux box without
 ¤ `proprietary` distribution dependent package manager (apt-get, rpm,
 ¤ pacman, pkgtool...)
 ¤
 ¤ usage:
 ¤ < sudo /usr/local/sbin/packmgn.sh |install|uninstall| /path_to*/file.tgz >
 ¤ < sudo /usr/local/sbin/packmgn.sh |-h|--help| >
 ¤
 ¤ * required, even if in the same directory than PackMgn.
 ¤
 ¤ Very Basic tool ! it don`t:
 ¤ - Handle another compression than gzip (neither *.tbz nor *.txz).
 ¤ - Manage dependencies.
 ¤ - Update.
 ¤ - Remove the eventual config file created by the program uninstalled.
 ¤ - Launch any script after install or uninstall.
 ¤ - Write any log.
 ¤
 ¤ Install operation replace all eventually existent files, regardless
 ¤ date, size or attributes.
 ¤
 ¤ For uninstallation, the original tarball must be present. PackMgn
 ¤ take care of your eventually modified files. It create a file list to
 ¤ suppress from the archive, compare these files with the installed files
 ¤ and remove it only if they are identical, (based on time, size,
 ¤ attributes).
 ¤
 ¤ Superuser rights are required for any (un)installation.
 ¤ PACKMGN should run under any Linux distro on x86 or ARM.
 ¤ Tested with Xinutop 2.0, based on Xfce Debian Jessie.
 ¤
 ¤ INSTALLATION: PackMgn is provided in a tar.gz archive (tgz), so simply
 ¤ decompress it:
 ¤    < sudo tar -xvz -C / -f packmgn-x.x.tgz >
 ¤ It will be installed in /usr/local/sbin
 ¤
 ¤ GPL 3+, beta software, use it under your own responsability !
 ¤ Frank Baudet <http://marinux.tuxfamily.org>
 ¤
   \n'

   printf "$help"
}

############################### END HELP ###############################


# for test
#destination=/tmp/
# real mode
destination=/


## cmdline help ##
if [ "$1" = "--help" ] || [ "$1" = "-h" ]; then
   disp_help | more
   exit


## need to be run as superuser ##
elif [ $(id -u) -ne 0 ]; then
   printf "\n   Error ! PACKMGN require superuser rights,
      type < packmgn.sh -h or --help > \n\n"
   exit


## incomplete cmd line ##
elif [ -z "$1" ] || [ -z "$2" ] ; then
   printf "\n   Error ! in command line
      type < packmgn.sh -h or --help > \n\n"
   exit


## install ##
elif [ "$1" = install ]; then
   echo
   tar -xvz --directory "$destination" -f "$2"
   [ $? -eq 0 ] && printf "\n   $2 unpacked to $destination \n\n"
   exit


## uninstall ##
elif [ "$1" = uninstall ]; then
   cd ${destination}
   echo
   # create list files to exclude
   touch /tmp/packmgn.excl

   # look for modified files
   tar -df "$2" | sort | while read all_dif
   do
      # remove comments
      fil_dif=$(echo "$all_dif" | cut -d ':' -f1)
      # show only files, not dirs
      if ! [ -d  "$destination"/"$fil_dif" ]; then
         echo "$all_dif"
         # create list
         echo "$fil_dif" >>/tmp/packmgn.excl
      fi
   done
   if [ -s /tmp/packmgn.excl ]; then
      printf "\n These file(s) are different(s) than the original(s)
         and will be preserved. \n\n"
      read -p "   hit <enter> to continue " cont
   fi

   # list all files in archive
   tar --list -f "$2" | sort -r | while read fil_dir
   do
      # compare and exclude modified files
      if ! (echo "$fil_dir" | grep -xcq -f /tmp/packmgn.excl); then
         # remove files
         if ! [ -d  "$fil_dir" ]; then
            echo "* FILE        $fil_dir"
            rm "$fil_dir"
         # remove empty dirs
         else
            rmdir "$fil_dir" 2>/dev/null
            [ $? = 0 ] && echo "* DIRECTORY   $fil_dir"
         fi
      fi
   done
   if [ "$fil_dir" != "" ]; then
      printf "\n   Are removed from $destination. \n\n"
      [ -f /tmp/packmgn.excl ] && rm /tmp/packmgn.excl
      exit
   fi

else
   printf "\n   Error ! type < packmgn.sh -h or --help > \n\n"
   exit

fi
